package com.persagy.dmp.rwd.dic;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.node.ArrayNode;
import com.fasterxml.jackson.databind.node.JsonNodeFactory;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.json.JacksonMapper;
import com.persagy.dmp.common.http.HttpUtils;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @author: yaoll
 * @date: 2020-11-30
 * @verison: 1.0
 */
@Component
public class FuncidUtils {

    private final static Map<String,String> baseFuncid = new HashMap<>();

    public static HttpUtils httpUtils;

    public static String rwdeditServer;

    static {
        baseFuncid.put("ProjID","id");
        baseFuncid.put("ProjName","name");
        baseFuncid.put("ProjLocalID","localId");
        baseFuncid.put("ProjLocalName","localName");
        baseFuncid.put("BuildID","id");
        baseFuncid.put("BuildName","name");
        baseFuncid.put("BuildLocalID","localId");
        baseFuncid.put("BuildLocalName","localName");
        baseFuncid.put("FloorID","id");
        baseFuncid.put("FloorName","name");
        baseFuncid.put("FloorLocalID","localId");
        baseFuncid.put("FloorLocalName","localName");
        baseFuncid.put("RoomID","id");
        baseFuncid.put("RoomName","name");
        baseFuncid.put("RoomLocalID","localId");
        baseFuncid.put("RoomLocalName","localName");
        baseFuncid.put("ShaftID","id");
        baseFuncid.put("ShaftName","name");
        baseFuncid.put("ShaftLocalID","localId");
        baseFuncid.put("ShaftLocalName","localName");
        baseFuncid.put("SysID","id");
        baseFuncid.put("SysName","name");
        baseFuncid.put("SysLocalID","localId");
        baseFuncid.put("SysLocalName","localName");
        baseFuncid.put("EquipID","id");
        baseFuncid.put("EquipName","name");
        baseFuncid.put("EquipLocalID","localId");
        baseFuncid.put("EquipLocalName","localName");
        baseFuncid.put("ToolID","id");
        baseFuncid.put("ToolName","name");
        baseFuncid.put("ToolLocalID","localId");
        baseFuncid.put("ToolLocalName","localName");
        baseFuncid.put("ConsumID","id");
        baseFuncid.put("ConsumName","name");
        baseFuncid.put("ConsumLocalID","localId");
        baseFuncid.put("ConsumLocalName","localName");

        baseFuncid.put("ECModelID","id");
        baseFuncid.put("ECModelName","name");
        baseFuncid.put("ECModelLocalID","localId");
        baseFuncid.put("ECModelLocalName","localName");

        baseFuncid.put("SubentryID","id");
        baseFuncid.put("SubentryName","name");
        baseFuncid.put("SubentryLocalID","localId");
        baseFuncid.put("SubentryLocalName","localName");

        baseFuncid.put("BranchID","id");
        baseFuncid.put("BranchName","name");
        baseFuncid.put("BranchLocalID","localId");
        baseFuncid.put("BranchLocalName","localName");

        baseFuncid.put("BeaconID","id");
        baseFuncid.put("BeaconlName","name");
        baseFuncid.put("BeaconLocalID","localId");
        baseFuncid.put("BeaconLocalName","localName");

        baseFuncid.put("TenantID","id");
        baseFuncid.put("TenantName","name");
        baseFuncid.put("TenantLocalID","localId");
        baseFuncid.put("TenantLocalName","localName");

    }

    public static boolean isBase(String code) {
        return baseFuncid.containsKey(code);
    }
    public static String getRealCode(String code) {
        return baseFuncid.get(code);
    }

    public static ArrayNode transEnumDataSource(String dataSource) {
        ArrayNode array = JsonNodeFactory.instance.arrayNode();
        if (dataSource == null) {
            return null;
        }
        // 包含换行符 直接返回
        if (dataSource.indexOf("\n") > -1) {
            return null;
        }
        // 预处理 ". " -> "."
        while (dataSource.indexOf(". ") > -1) {
            dataSource = dataSource.replace(". ", ".");
        }
        if (dataSource.contains("《")) {
            String name = null;
            String refKey = null;
            Pattern pattern = Pattern.compile("《(.*?)》");
            Matcher matcher = pattern.matcher(dataSource);
            while (matcher.find()) {
                String group = matcher.group(1);
                if ("4天气状态".equals(group)) {
                    name = "天气状态";
                } else if ("6方向代码".equals(group)) {
                    name = "方向代码";
                } else if ("10抗震设防烈度".equals(group)) {
                    name = "抗震设防烈度";
                } else {
                    name = group;
                }
            }
            if ("见《推荐空间业态分类表》-项目类型".equals(dataSource)) {
                name = "项目类型";
            } else if ("见《枚举值释义》方向代码".equals(dataSource)) {
                name = "方向代码";
            } else if ("参考《11防护等级》".equals(dataSource)) {
                // 预处理
                String tmp = dataSource.replace("（多选）", "")
                        .replace("（可多选）", "")
                        .replace("（可以多选）", "");

                // 尝试解析
                // 1.铸铁 2.钢 3.铝合金 4.铜管 5.复合型 99.其他
                String[] split = tmp.split(" ");
                if (split.length == 0) {
                    return null;
                }
                Map<String, String> data = new HashMap<>();
                for (int i = 0; i < split.length; i++) {
                    String temp = split[i];
                    int idx = temp.indexOf(".");
                    if (idx == -1) {
                        continue;
                    }
                    String key = temp.substring(0, idx);
                    String value = temp.substring(idx + 1);
                    data.put(key, value);
                    ObjectNode item = array.addObject();
                    item.put("code", key);
                    item.put("name", value);
                }
                return array;
            }

            //查询知识库引用
            String url = rwdeditServer + "/rwdedit/ref/type/query";
            JacksonCriteria criteria = JacksonCriteria.newInstance();
            criteria.add("name", name);
            HttpUtils.HttpResult response = httpUtils.post(url, JacksonMapper.toSimpleJson(criteria), 300000);
            if (!response.success()) {
                return null;
            }
            String content = response.getContent();
            ObjectNode objectNode = JacksonMapper.toObject(content, ObjectNode.class);
            ArrayNode data = JacksonMapper.getArray(objectNode, "data", JsonNodeFactory.instance.arrayNode());
            refKey = JacksonMapper.getString((ObjectNode) data.get(0), "refKey");
            if (null == refKey) {
                return null;
            }

            //查询知识库详情
            url = rwdeditServer + "/rwdedit/ref/type/infos/query";
            criteria = JacksonCriteria.newInstance();
            criteria.add("refKey", refKey);
            response = httpUtils.post(url, JacksonMapper.toSimpleJson(criteria), 300000);
            if (!response.success()) {
                return null;
            }
            content = response.getContent();
            objectNode = JacksonMapper.toObject(content, ObjectNode.class);
            data = JacksonMapper.getArray(objectNode, "data", JsonNodeFactory.instance.arrayNode());
            for (JsonNode datum : data) {
                ObjectNode obj = (ObjectNode) datum;
                ObjectNode item = array.addObject();
                item.put("code", JacksonMapper.getString(obj, "code"));
                item.put("name", JacksonMapper.getString(obj, "name"));
                String parentCode = JacksonMapper.getString(obj, "parentCode");
                if (parentCode != null && !("-9999".equals(parentCode))) {
                    item.put("parentCode", parentCode);
                }
            }
        } else if (dataSource.indexOf(".") > -1) {

            // 预处理
            String tmp = dataSource.replace("（多选）", "")
                    .replace("（可多选）", "")
                    .replace("（可以多选）", "");

            // 尝试解析
            // 1.铸铁 2.钢 3.铝合金 4.铜管 5.复合型 99.其他
            String[] split = tmp.split(" ");
            if (split.length == 0) {
                return null;
            }
            Map<String, String> data = new HashMap<>();
            for (int i = 0; i < split.length; i++) {
                String temp = split[i];
                int idx = temp.indexOf(".");
                if (idx == -1) {
                    continue;
                }
                String key = temp.substring(0, idx);
                String value = temp.substring(idx + 1);
                data.put(key, value);
                ObjectNode item = array.addObject();
                item.put("code", key);
                item.put("name", value);
            }
        }
        return array;
    }
}
